/*
 * ChessCanvas.java
 *
 * Created on 1. Dezember 2007, 22:07
 */

/*
 The contents of this file are subject to the
 Mozilla Public License Version 1.1 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied.
 See the License for the specific language governing rights
 and limitations under the License.

 The Initial Developer of the Original Code is
 Georg-Johann Lay (GeorgJohann@web.de).

 Portions created by the Initial Developer are
 Copyright (C) 2007 Georg-Johann Lay. All Rights Reserved.

 Alternatively, the contents of this file may be used under the terms
 of either the GNU General Public License Version 3 or later (the "GPL"),
 or the GNU Lesser General Public License Version 3 or later (the "LGPL"),
 in which case the provisions of the GPL or the LGPL are applicable instead
 of those above.

 If you wish to allow use of your version of this file only
 under the terms of either the GPL or the LGPL, and not to allow others to
 use your version of this file under the terms of the MPL, indicate your
 decision by deleting the provisions above and replace them with the notice
 and other provisions required by the LGPL or the GPL. If you do not delete
 the provisions above, a recipient may use your version of this file under
 the terms of any one of the MPL, the GPL or the LGPL.
*/

package de.gjlay.chess;

import de.gjlay.mobile.localize.*;
import de.gjlay.mobile.ui.*;
import javax.microedition.lcdui.*;
import javax.microedition.midlet.MIDlet;

/**
 * A Canvas that is capable of painting a chess board and chess tiles.
 * @see Board
 * @author Georg-Johann Lay
 */
abstract class ChessCanvas
        extends Canvas
        implements BOARD_CONST, COLOR_CONST, CoDec
{
    static final int SIZE = 21;

    private static final String LOVER_ID = "drawer";

    private final static int INTERVAL = 700;

    private Thread blinkThread;
    private int blink;
    protected int cursorCount;

    static final int CURSOR_MARK  = 1;
    static final int CURSOR_BLINK = 2;
    static final int CURSOR_MARKBLINK  = 3;

    static final int M_VOID  = -1;
    static final int M_FIX   = 0;
    static final int M_TURN2 = 1;
    static final int M_FIX2  = 2;
    static final int M_FLIP2 = 3;

    int mode = M_FIX;
    boolean turn      = false; // board
    boolean flipBlack = false; // pieces
    boolean flipWhite = false; // pieces
    boolean showLetters = true;
    static boolean BacklightOn = false;

    protected Board drawBoard;
    protected int drawMover;
    protected Graphics drawGraph;

    protected static Tiles chessTiles;
    protected static CharTiles fontTiles;
    protected static Image lightImage;

    // 6*3*(21*21)
    private static final String CHESS_TILES = "/res/images/chess-tiles.png";
    private static final String LIGHT_IMAGE = "/res/images/light.png";

    static MIDlet midlet;
    static
    {
        try
        {
            chessTiles = new Tiles (Image.createImage (CHESS_TILES), SIZE, SIZE);
            fontTiles = new Font15();
            lightImage = Image.createImage (LIGHT_IMAGE);
        }
        catch (Exception e)
        {
            //e.printStackTrace ();
        }
    }

    ChessCanvas (MIDlet midlet)
    {
        this.midlet = midlet;
        blinkThread = getBlinkThread();
        blinkThread.setPriority (Thread.MIN_PRIORITY);
        blinkThread.start();
    }

    protected void setDrawContext (Graphics graph, Board board, int mover)
    {
        drawGraph = graph;
        drawBoard = board;
        drawMover = mover;
    }

    protected void drawBackground (int color)
    {
        drawGraph.setColor (color);
        drawGraph.fillRect (0, 0, getWidth(), getHeight());
    }
    /////////////////////////////////////////////////////////////
    protected void drawFrame ()
    {
        int width = 5;

        if (showLetters)
        {
            width = Math.max (fontTiles.getSizeX(), fontTiles.getSizeY()) -4;
            drawSquare (C_BORDER_BACK,  width, true);
            drawLetters (width);
            int x, y, r = 2;
            int w = width-2-2*r;

            x = -width+r;
            y = 8*SIZE+1+r;

            drawGraph.setColor (C_BORDER_FRAME);
            drawGraph.drawRect (x, y, w, w);
            drawGraph.drawRect (y, x, w, w);
            drawGraph.setColor ((drawMover == WHITE) ? 0xffffff : 0x404040);
            drawGraph.fillRect (x+1, y+1, w-1, w-1);
            drawGraph.fillRect (y+1, x+1, w-1, w-1);
            if (ChessCanvas.BacklightOn)
                drawGraph.drawImage (lightImage, 8*SIZE+width/2, 8*SIZE+width/2-2, drawGraph.HCENTER | drawGraph.VCENTER);
            
            Move m = drawBoard.h.getActualMove();
            
            if (null != m && 0 != m.rating)
            {
                drawGraph.setColor (C_BORDER_FRAME);
                drawGraph.fillRect (-width, -width, width, width);
                
                int i = m.rating;
                drawGraph.setColor (i < 0 ? 0xff0000 : 0x00e000);
                i = Math.abs(i);
                
                int p = -width+width/2-2*i-1;
                drawGraph.fillRect (p, p, 1+4*i, 1+4*i);
            }
        }
        else
        {
            drawSquare (C_BORDER_BACK, width, true);

            int sx = 6;
            drawGraph.setColor (C_BORDER_FRAME);
            drawGraph.drawRect (4*SIZE-sx/2-1, 8*SIZE,   sx+1, width);
            drawGraph.drawRect (4*SIZE-sx/2-1, -width-1, sx+1, width);
            drawGraph.drawRect (-width-1, 8*SIZE, width, width);
            
            if (ChessCanvas.BacklightOn)
            {
                drawGraph.drawRect (8*SIZE, 8*SIZE, width, width);
                drawGraph.setColor (C_LIGHT);
                drawGraph.fillRect (8*SIZE+1, 8*SIZE+1, width-1, width-1);
            }

            drawGraph.setColor ((drawMover == WHITE) ? C_MOVER_WHITE : C_MOVER_BLACK);
            drawGraph.fillRect (-width, 8*SIZE+1, width-1, width-1);

            drawGraph.setColor (turn ? C_MOVER_BLACK : C_MOVER_WHITE);
            drawGraph.fillRect (4*SIZE-sx/2, 8*SIZE+1, sx, width-1);

            drawGraph.setColor (turn ? C_MOVER_WHITE : C_MOVER_BLACK);
            drawGraph.fillRect (4*SIZE-sx/2, -width, sx, width-1);
        }

        drawSquare (C_BORDER_FRAME, width+1, false);
        drawSquare (C_BORDER_FRAME, 1, false);
    }

    protected void drawLetters (int width)
    {
        int c, xx, yy, anchor = drawGraph.HCENTER | drawGraph.VCENTER;
        boolean flipBottom = false;
        boolean flipTop = mode == M_FIX2 || mode == M_FLIP2;

        for (int x = 0; x < 8; x++)
        {
            c = 'A' + (turn ? 7-x : x);
            // unten
            //flipped = flipWhite && flipBlack;
            xx = SIZE*x+SIZE/2;
            yy = 8*SIZE + width/2 + (flipBottom ? 0 : 1);
            fontTiles.drawChar (drawGraph, c, xx, yy, flipBottom, anchor);

            // oben
            //flipped = flipWhite || flipBlack;
            yy = -width/2 + (flipTop ? -1 : 0);
            fontTiles.drawChar (drawGraph, c, xx, yy, flipTop, anchor);
        }

        for (int y = 0; y < 8; y++)
        {
            c = '1' + (turn ? 7-y : y);
            // links
            //flipped = flipWhite && flipBlack;
            xx = -width/2;
            yy = SIZE*(7-y) + SIZE/2 + (flipBottom ? 0 : 1);
            fontTiles.drawChar (drawGraph, c, xx, yy, flipBottom, anchor);

            // rechts
            //flipped = flipWhite || flipBlack;
            xx = 8*SIZE + width/2;
            yy = SIZE*(7-y) + SIZE/2 + (flipTop ? 0 : 1);
            fontTiles.drawChar (drawGraph, c, xx, yy, flipTop, anchor);
        }
    }

    protected void drawSquare (int color, int r, boolean fill)
    {
        drawGraph.setColor (color);

        if (fill)
            drawGraph.fillRect (-r, -r, 8*SIZE+2*r, 8*SIZE+2*r);
        else
            // DRAW-RECT SIZE -=1 damit = fill-rect
            drawGraph.drawRect (-r, -r, 8*SIZE+2*r -1, 8*SIZE+2*r -1);
    }

    /////////////////////////////////////////////////////////////
    protected void drawBoard ()
    {
        for (int x=0; x<8; x++)
        for (int y=0; y<8; y++)
        {
            drawGraph.setColor (((x+y) % 2 == 0) ? C_FIELD_BLACK : C_FIELD_WHITE);
            drawGraph.fillRect (SIZE*x, SIZE*(7-y), SIZE, SIZE);
        }
    }

    protected void drawPieces ()
    {
        for (int x=0; x<8; x++)
        for (int y=0; y<8; y++)
        {
            int color = drawBoard.c (x,y);
            int id = GetTileIndex (drawBoard.p(x,y), color);

            if (id < 0)
                continue;

            int yy = turn ? y : 7-y;
            int xx = turn ? 7-x : x;
            boolean flipped = (flipWhite && color == WHITE)
                    || (flipBlack && color == BLACK);
            chessTiles.paint (drawGraph, id, SIZE*xx, SIZE*yy, flipped, 0);
        }
        // Schach ?
        //if (Drawer.canvas == ChessCanvas.getInstance() && drawBoard.isCheck())
          //  drawCheck (drawBoard.mover);
    }

    protected void drawCheck (int color)
    {
        if (0 == blink || !drawBoard.canMove)
            drawKingHilit (color, C_KING_CHECK);
    }

    private void drawKingHilit (int color, int theColor)
    {
        Field king = drawBoard.king[color];
        if (null == king)
            return;
        int yy = turn ? king.y : 7-king.y;
        int xx = turn ? 7-king.x : king.x;

        boolean flipped = (flipWhite && color == WHITE)
                ||(flipBlack && color == BLACK);
        xx = SIZE*xx + 10;
        yy = SIZE*yy + (flipped ? 12 : 7);

        drawGraph.setColor (theColor);
        drawGraph.drawLine (xx, yy, xx, yy+1);
    }

    private static int GetTileIndex (int id, int color)
    {
        if (EMPTY == id)
            return -1;

        return (id >> 1) + 6*color -1;
    }

    /////////////////////////////////////////////////////////////
    protected void drawCursor (int style, int x, int y)
    {
        int yy = SIZE * (turn ? y : 7-y);
        int xx = SIZE * (turn ? 7-x : x);

        if (style == CURSOR_MARK
                || (style == CURSOR_MARKBLINK && blink == 0))
            chessTiles.paint (drawGraph, 12 + drawMover, xx, yy, false, 0);

        if (style == CURSOR_BLINK)
            chessTiles.paint (drawGraph, (1==blink) ? 17 : 14+drawMover, xx, yy, false, 0);

        if (style == CURSOR_MARKBLINK && blink == 1)
            chessTiles.paint (drawGraph, 16, xx, yy, false, 0);

        if (style <= 0)
        {

            if (0 == blink)
            {
                int piece = -style;
                int id = GetTileIndex (piece & ~1, piece & 1);

                drawCursor (CURSOR_MARKBLINK, x, y);
                drawGraph.setColor (((x+y) % 2 == 0) ? C_FIELD_BLACK : C_FIELD_WHITE);
                drawGraph.fillRect (xx+2, yy+2, SIZE-4, SIZE-4);

                chessTiles.paint (drawGraph, id, xx, yy, false, 0);
            }
        }
    }

    private Thread getBlinkThread()
    {
        Thread thread = new Thread (new Runnable()
        {
            public void run() {
                while (true) {
                    try { Thread.sleep (50); }
                    catch (InterruptedException ex) {}

                    try {
                        if (null != midlet && ChessCanvas.BacklightOn)
                            Display.getDisplay (midlet).flashBacklight (1000);
                    } catch (Exception e) {}

                    synchronized (this) {
                        blink = cursorCount / (INTERVAL/50);
                        if (0 == cursorCount % (INTERVAL/50))
                            repaint();
                        cursorCount = (1+cursorCount) % (2*INTERVAL/50);
        }}}});

        return thread;
    }

    protected void addMode (int newMode, int mover)
    {
        turn = (newMode == mode) ? !turn : false;
        mode = newMode;
        flushMode (mover);
        flushMode (mover);
    }

    protected void flushMode (int mover)
    {
        switch (mode)
        {
            default:
                break;

            case M_TURN2:
                turn = !turn;
            case M_FIX:
                flipBlack = flipWhite = false;
                break;

            case M_FIX2:
                flipWhite = turn;
                flipBlack = !turn;
                break;

            case M_FLIP2:
                flipWhite = (mover == BLACK) ^ turn;
                flipBlack = flipWhite;
                break;
        }
    }

    ////////////////////////////////////////////////////////////////

    void save (String name)
    {
        Lover.Save (name+LOVER_ID, this);
    }

    void load (String name, int mover)
    {
        byte[] code = Lover.Load (name+LOVER_ID);

        if (null == code)
        {
            addMode (M_VOID, mover);
            addMode (M_FIX , mover);
        }
        else
            decode (code, new Integer (mover));
    }
    ////////////////////////////////////////////////////////////////

    public int getEncodeSize()
    {
        return 5;
    }

    public byte[] encode ()
    {
        byte[] code = new byte[getEncodeSize()];

        code[0] = (byte) mode;
        code[1] = (byte) (turn        ? 1 : 0);
        code[2] = (byte) (flipWhite   ? 1 : 0);
        code[3] = (byte) (flipBlack   ? 1 : 0);
        code[4] = (byte) (showLetters ? 1 : 0);

        return code;
    }

    public void decode (byte[] code, Object mover)
    {
        mode        = code[0];
        turn        = code[1] == 1;
        flipWhite   = code[2] == 1;
        flipBlack   = code[3] == 1;
        showLetters = code[4] == 1;

        flushMode (((Integer) mover).intValue());
        flushMode (((Integer) mover).intValue());
    }

}
