/*
 * Localizer.java
 *
 * Created on 26. November 2007, 19:57
 *
 */

/*
 The contents of this file are subject to the
 Mozilla Public License Version 1.1 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied.
 See the License for the specific language governing rights
 and limitations under the License.

 The Initial Developer of the Original Code is
 Georg-Johann Lay (GeorgJohann@web.de).

 Portions created by the Initial Developer are
 Copyright (C) 2007 Georg-Johann Lay. All Rights Reserved.

 Alternatively, the contents of this file may be used under the terms
 of either the GNU General Public License Version 3 or later (the "GPL"),
 or the GNU Lesser General Public License Version 3 or later (the "LGPL"),
 in which case the provisions of the GPL or the LGPL are applicable instead
 of those above.

 If you wish to allow use of your version of this file only
 under the terms of either the GPL or the LGPL, and not to allow others to
 use your version of this file under the terms of the MPL, indicate your
 decision by deleting the provisions above and replace them with the notice
 and other provisions required by the LGPL or the GPL. If you do not delete
 the provisions above, a recipient may use your version of this file under
 the terms of any one of the MPL, the GPL or the LGPL.
*/

package de.gjlay.mobile.localize;

import de.gjlay.chess.*;

/**
 *
 * Manages changes of locale and informs its listeners.
 * @author Georg-Johann Lay
 */
public interface Localizer
{
    /**
     * Add a LanguageListener which shall be informed on changes of
     * locale, i.e. calls to {@link #setLocale(String)}
     * @param languageListener LanguageListener to add to the list
     */
    public void addLanguageListener (LanguageListener languageListener);

    /**
     * Set locale like <code>"de"</code> or <code>"en-US"</code>.
     * If locale is <code>null</code> then locale will be set to
     * <code>System.getProperty ("microedition.locale")</code>.
     * @param locale the locale to set like <CODE>"en_US"</CODE> or <CODE>"fr"</CODE>.
     * Passing <CODE>null</CODE> will set the platforms default language
     * as specified by the system property <CODE>"microedition.locale"</CODE>.
     * @see java.lang.System#getProperty(String)
     */
    public void setLocale (String locale);

    /**
     * Get locale
     * according (not neccessarily equal) to the last call of
     * {@link #setLocale(String)}.
     * @return the locale
     */
    public String getLocale ();
}
