/*
 * Selector.java
 *
 * Created on 27. November 2007, 01:33
 *
 */

/*
 The contents of this file are subject to the
 Mozilla Public License Version 1.1 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied.
 See the License for the specific language governing rights
 and limitations under the License.

 The Initial Developer of the Original Code is
 Georg-Johann Lay (GeorgJohann@web.de).

 Portions created by the Initial Developer are
 Copyright (C) 2007 Georg-Johann Lay. All Rights Reserved.

 Alternatively, the contents of this file may be used under the terms
 of either the GNU General Public License Version 3 or later (the "GPL"),
 or the GNU Lesser General Public License Version 3 or later (the "LGPL"),
 in which case the provisions of the GPL or the LGPL are applicable instead
 of those above.

 If you wish to allow use of your version of this file only
 under the terms of either the GPL or the LGPL, and not to allow others to
 use your version of this file under the terms of the MPL, indicate your
 decision by deleting the provisions above and replace them with the notice
 and other provisions required by the LGPL or the GPL. If you do not delete
 the provisions above, a recipient may use your version of this file under
 the terms of any one of the MPL, the GPL or the LGPL.
*/

package de.gjlay.mobile.ui;

import de.gjlay.chess.*;
import de.gjlay.mobile.localize.L;
import javax.microedition.midlet.MIDlet;

import javax.microedition.lcdui.*;

/**
 *
 * @author Georg-Johann Lay
 */
public final class Selector
    extends List
        implements CommandListener
{
    public static final int BACK = -1;

    private MIDlet midlet;

    private Command backCommand = null;
    private Command itemCommand = null;
    private SelectListener listener;
    private SelectorLabel[] labels;
    private int tag;

    private static Selector instance;

    private Selector (MIDlet midlet)
    {
        super (null, List.IMPLICIT);
        this.midlet    = midlet;
        init();
    }

    public static Selector GetInstance (MIDlet midlet)
    {
        return
            instance = new Selector (midlet);
    }
    
    public static int GetSelectedIndex()
    {
        return instance.getSelectedIndex();
    }
    
    public static void SetSelectedIndex (int index)
    {
        instance.setSelectedIndex (index, true);
    }

    private void init()
    {
        setSelectCommand (itemCommand = new Command ("", Command.ITEM, 3));
        setCommandListener (this);
    }

    public static void Show (SelectListener listener, int tag, String title, SelectorLabel[] labels)
    {
        instance.listener = listener;
        instance.labels   = labels;
        instance.tag      = tag;

        instance.show (title);
    }

    private void show (String title)
    {
        setTitle (title);

        removeCommand (backCommand);
        addCommand (backCommand = new Command (L._("T_BACK"), Command.BACK, 2));
        
        deleteAll();

        for (int i=0; i < labels.length; i++)
            append (labels[i].label, null);

        Display.getDisplay (midlet).setCurrent (this);
    }

    public void commandAction (Command command, Displayable displayable)
    {
        if (command == backCommand)
            listener.selectionDone (BACK, tag, -1);

        if (command == itemCommand)
            listener.selectionDone (labels[getSelectedIndex()].id, tag, getSelectedIndex());
    }
}
